==> 1.0.0.tar.gz: lib/vibdata_processing.dart <==
/*
 * @Author: renjie.yin
 * @Date: 2022-05-31 21:19:15
 * @LastEditors: renjie.yin
 * @LastEditTime: 2022-05-31 21:23:05
 * @Description: 
 */
import 'dart:math' as math;
import './signal_processing.dart';

List<double> dataIntegralFreqOne(
    List<double> data, double fs, double lowerFreq, double upperFreq) {
  var len = data.length;

  var fx = realFFT(data);

  var df = fs / len;

  var dw = 2 * math.pi * df;

  var w = List<double>.filled(len, 0);
  for (var i = 0; i < len; i++) {
    if (i <= len / 2 - 1) {
      w[i] = i == 0 ? 0 : w[i - 1] + dw;
    } else {
      w[i] = i == len / 2 ? (-len * dw) / 2 : w[i - 1] + dw;
    }
  }

  var fy1 = ArrayComplex.fixed(len);

  for (var i = 0; i < len; i++) {
    if (i == 0) {
      fy1[i] = Complex.ri(0, 0);
    } else {
      fy1[i] = fx[i].div(Complex.ri(0, w[i]));
    }
  }

  var iL = (lowerFreq / df).floor();
  var iH = (upperFreq / df).ceil();

  var fy2 = ArrayComplex.fixed(len);
  for (var i = 0; i < len; i++) {
    if (i >= iL && i <= iH) {
      fy2[i] = Complex.ri(fy1[i].real, fy1[i].imaginary);
    } else if (i >= len - iH && i <= math.min(len - 1, len - iL)) {
      fy2[i] = fy1[i];
    } else {
      fy2[i] = Complex.ri(0, 0);
    }
  }
  var outData = inverseRealFft(fy2);
  return outData;
}

class VibDataProcessing {
  //  加速度转速度
  static List<double> accToVel(
      List<double> wave, double fs, double fMin, double fMax) {
    return dataIntegralFreqOne(wave, fs, fMin, (fMax * 2.56) / 2);
  }

// 加速度转位移
  static List<double> accToDist(
      List<double> wave, double fs, double fMin, double fMax) {
    var vel = accToVel(wave, fs, fMin, fMax);
    return dataIntegralFreqOne(vel, fs, fMin, (fMax * 2.56) / 2);
  }

// 去均值
  static List<double> removeArrayMean(List<double> data) {
    double total = 0;
    for (var element in data) {
      total += element;
    }
    var avg = total / data.length;
    return data.map((e) {
      return e - avg;
    }).toList();
  }

//填充数组
  static List<double> padWithZeros(List<double> data) {
    var dataLength = data.length;
    if (isPowerOf2(dataLength)) {
      return data;
    }
    // 将数据长度调整为2的N次方
    int nY = (math.log(dataLength) / math.log(2.0)).ceil();
    // const nY = ConvertToInt32(Math.ceil(Math.log(dataLength) / Math.log(2)));

    int userDataLength = math.pow(2, nY).floor();
    var userData = List<double>.filled(userDataLength, 0);
    for (var i = 0; i < dataLength; i++) {
      userData[i] = data[i];
    }
    return userData;
  }

// fft
  static List<double> fftCalc(List<double> data,
      {bool removeMean = true, bool paddingWithZeros = true}) {
    var calcData = data;
    if (removeMean) {
      calcData = removeArrayMean(calcData);
    }
    if (paddingWithZeros) {
      calcData = padWithZeros(calcData);
    }

    var cRes = realFFT(calcData);
    var d = cRes.map((item) {
      return math.sqrt(item.real * item.real + item.imaginary * item.imaginary);
    }).toList();
    var retLen = (calcData.length / 2.56).floor();
    return d.take(retLen).toList();
  }

// 获取波形的x轴间隔
  static double getWaveDataXInterval(double fs) {
    return 1 / fs * 1000;
  }

// 获取频谱的x轴间隔
  static double getSpecDataXInterval(List<double> data, double fs) {
    return fs / 2.56 / data.length;
  }
}

==> 1.0.1.tar.gz: lib/vibdata_processing.dart <==
/*
 * @Author: renjie.yin
 * @Date: 2022-05-31 21:19:15
 * @LastEditors: renjie.yin
 * @LastEditTime: 2022-06-07 10:34:07
 * @Description: 
 */
import 'dart:math' as math;
import 'dsp.dart';

List<double> dataIntegralFreqOne(
    List<double> data, double fs, double lowerFreq, double upperFreq) {
  var len = data.length;

  var fx = realFFT(data);

  Stopwatch stopwatch = Stopwatch()..start();

  var df = fs / len;

  var dw = 2 * math.pi * df;

  var w = List<double>.filled(len, 0);
  for (var i = 0; i < len; i++) {
    if (i <= len / 2 - 1) {
      w[i] = i == 0 ? 0 : w[i - 1] + dw;
    } else {
      w[i] = i == len / 2 ? (-len * dw) / 2 : w[i - 1] + dw;
    }
  }

  var fy1 = ArrayComplex.fixed(len);

  for (var i = 0; i < len; i++) {
    if (i == 0) {
      fy1[i] = Complex.ri(0, 0);
    } else {
      fy1[i] = fx[i].div(Complex.ri(0, w[i]));
    }
  }

  var iL = (lowerFreq / df).floor();
  var iH = (upperFreq / df).ceil();

  var fy2 = ArrayComplex.fixed(len);
  for (var i = 0; i < len; i++) {
    if (i >= iL && i <= iH) {
      fy2[i] = Complex.ri(fy1[i].real, fy1[i].imaginary);
    } else if (i >= len - iH && i <= math.min(len - 1, len - iL)) {
      fy2[i] = fy1[i];
    } else {
      fy2[i] = Complex.ri(0, 0);
    }
  }
  print('dataIntegralFreqOne() executed in ${stopwatch.elapsed}');
  var outData = inverseRealFft(fy2);
  return outData;
}

class VibDataProcessing {
  //  加速度转速度
  static List<double> accToVel(
      List<double> wave, double fs, double fMin, double fMax) {
    return dataIntegralFreqOne(wave, fs, fMin, (fMax * 2.56) / 2);
  }

// 加速度转位移
  static List<double> accToDist(
      List<double> wave, double fs, double fMin, double fMax) {
    var vel = accToVel(wave, fs, fMin, fMax);
    return dataIntegralFreqOne(vel, fs, fMin, (fMax * 2.56) / 2);
  }

// 去均值
  static List<double> removeArrayMean(List<double> data) {
    double total = 0;

    for (var element in data) {
      total += element;
    }
    var avg = total / data.length;
    return data.map((e) {
      return e - avg;
    }).toList();
  }

  //填充数组长度到2^n
  static List<double> padArrayWithZeros(List<double> data) {
    var dataLength = data.length;
    if (isPowerOf2(dataLength)) {
      return data;
    }
    int nY = (math.log(dataLength) / math.log(2.0)).ceil();
    int userDataLength = math.pow(2, nY).floor();
    var userData = List<double>.filled(userDataLength, 0);
    for (var i = 0; i < dataLength; i++) {
      userData[i] = data[i];
    }
    return userData;
  }

  static List<double> fftCalc(List<double> data,
      {bool removeMean = true, bool paddingWithZeros = true}) {
    var calcData = data;
    if (removeMean) {
      calcData = removeArrayMean(calcData);
    }
    if (paddingWithZeros) {
      calcData = padArrayWithZeros(calcData);
    }
    var fftResult = realFFT(calcData);
    var retLen = (calcData.length / 2.56).floor();
    return fftResult.take(retLen).map((item) {
      return item.magnitude;
    }).toList();
  }

// 获取波形的x轴间隔
  static double getWaveDataXInterval(double fs) {
    return 1 / fs * 1000;
  }

// 获取频谱的x轴间隔
  static double getSpecDataXInterval(List<double> data, double fs) {
    return fs / 2.56 / data.length;
  }
}
